﻿// Sketchables.NET - Design Time Support Libraries
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Gnu General Public License (GPL) v3
// Please see http://www.gnu.org/licenses/gpl.html for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Windows.Design.Interaction;
using Microsoft.Windows.Design.Model;
using Sketchables.Design.Markup;

namespace Sketchables.Design.ContentEditor
{
  internal class InPlaceEditorProvider : PrimarySelectionAdornerProvider
  {
    private ModelItem modelItem;
    private SketchableTypeFactory typeFactory;

    protected ModelItem AdornedModelItem
    {
      get { return modelItem; }
      set
      {
        if (modelItem != null)
        {
          modelItem.PropertyChanged -= OnModelItemPropertyChanged;
        }

        modelItem = value;
        if (modelItem != null)
        {
          modelItem.PropertyChanged += OnModelItemPropertyChanged;
        }
      }
    }

    protected AdornerPanel AdornerPanel { get; set; }

    /// <summary>
    /// Toolbox that is being displayed above the selected control.
    /// </summary>
    protected SketchableToolbarAdorner ToolbarAdorner { get; set; }

    /// <summary>
    /// The text editor that provides editing for the entered content.
    /// </summary>
    protected SketchableEditor Editor { get; set; }


    public InPlaceEditorProvider()
    {
      typeFactory = Ioc.Container.Get<SketchableTypeFactory>();
    }


    private void InitToolBoxAdorner()
    {
      if(ToolbarAdorner != null)
      {
        //make sure toolbar is not collapsed
        ToolbarAdorner.Visibility = Visibility.Visible;
        return;
      }

      ToolbarAdorner = new SketchableToolbarAdorner {Cursor = Cursors.Hand};
      ToolTipService.SetToolTip(ToolbarAdorner, "Click to edit Sketch markup.");
      
      // Setup the top AdornerPanel.
      // All adorners are placed in an AdornerPanel
      // for sizing and layout support.
      AdornerPanel = new AdornerPanel();

      AdornerPanel.IsContentFocusable = true;
      AdornerPanel.Children.Add(ToolbarAdorner);
      Adorners.Add(AdornerPanel);

      AdornerPanel.SetHorizontalStretch(ToolbarAdorner, AdornerStretch.None);
      AdornerPanel.SetVerticalStretch(ToolbarAdorner, AdornerStretch.None);

      //place adorner on top left corner of the adorned control,
      //and move it 5 pixels up
      var topPlacements = new AdornerPlacementCollection();
      topPlacements.SizeRelativeToContentWidth(1.0, 0.0);
      topPlacements.SizeRelativeToAdornerDesiredHeight(1, 0.0);
      topPlacements.PositionRelativeToAdornerHeight(-1.0, -5.0);

      AdornerPanel.SetPlacements(ToolbarAdorner, topPlacements);

      // Add interaction to the glyph.  A click starts in-place editing.
      ToolbarAdorner.Edit += ToggleEditing;
    }


    
    private void InitEditor()
    {
      if (Editor != null) return;
      Editor = new SketchableEditor {Visibility = Visibility.Collapsed};

      AdornerPanel.Children.Add(Editor);
      AdornerPanel.SetHorizontalStretch(Editor, AdornerStretch.Stretch);
      AdornerPanel.SetVerticalStretch(Editor, AdornerStretch.Stretch);

      var stretchPlacements = new AdornerPlacementCollection();

      //make adorner size the same as the content of the selected control
      stretchPlacements.SizeRelativeToContentWidth(1, 0);
      stretchPlacements.SizeRelativeToContentHeight(1, 0);
      AdornerPanel.SetPlacements(Editor, stretchPlacements);

      //register event listeners
      Editor.LostFocus += (s, e) => CommitChangesAndHideEditor();
      Editor.GotFocus += (s, e) => ToolbarAdorner.Visibility = Visibility.Collapsed;
      Editor.CommitChanges += CommitChangesAndHideEditor;
    }


    protected override void Activate(ModelItem item)
    {
      InitToolBoxAdorner();
      AdornedModelItem = item;
      base.Activate(item);
    }

    protected override void Deactivate()
    {
      AdornedModelItem = null;
      base.Deactivate();
    }


    private void ToggleEditing()
    {
      InitEditor();

      var content = ResolveConverter().GetMarkup(AdornedModelItem);
      Editor.EditorText = content;
      Editor.Visibility = Editor.IsVisible ? Visibility.Collapsed : Visibility.Visible;
    }


    /// <summary>
    /// Gets a converter for the selected item.
    /// </summary>
    /// <returns></returns>
    private ISketchingMarkupConverter ResolveConverter()
    {
      return typeFactory.GetConverter(AdornedModelItem);
    }


    /// <summary>
    /// Updats the model item and collapses the editor control.
    /// </summary>
    private void CommitChangesAndHideEditor()
    {
      try
      {
        if (AdornedModelItem != null)
        {
          ResolveConverter().UpdateSketchObject( AdornedModelItem, Editor.EditorText);
        }
        else
        {
          //TODO dummy code
          MessageBox.Show("No model item anymore (editor lost focus event listener)...");
        }

        //hide editor
        Editor.Visibility = Visibility.Collapsed;
        ToolbarAdorner.Visibility = Visibility.Visible;
      }
      catch (Exception exception)
      {
        MessageBox.Show("Design time exception during editor lost focus: " + exception.ToString());
      }
    }



    protected virtual void OnModelItemPropertyChanged(object sender, PropertyChangedEventArgs e)
    {
    }
  }
}