﻿// Sketchables.NET - Design Time Support Libraries
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Gnu General Public License (GPL) v3
// Please see http://www.gnu.org/licenses/gpl.html for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE



using System;
using System.Linq.Expressions;
using Microsoft.Windows.Design.Metadata;
using Microsoft.Windows.Design.Model;

namespace Sketchables.Design
{
  internal static class Extensions
  {
    public static string GetPropertyName<T>(Expression<Func<T, object>> expression)
    {
      Expression body = expression.Body;
      var memberExpression = body as MemberExpression;
      if (memberExpression == null)
      {
        memberExpression = (MemberExpression) ((UnaryExpression) body).Operand;
      }

      return memberExpression.Member.Name;
    }


    public static void SetCollectionPropertyValue<T>(this ModelItem item, Expression<Func<T, object>> expression, object value)
    {
      string propertyName = GetPropertyName(expression);
      item.Properties[propertyName].Collection.Add(value);
    }


    /// <summary>
    /// Gets a property for the underlying model using a strongly
    /// typed expression.
    /// </summary>
    public static ModelProperty GetProperty<T>(this ModelItem item, Expression<Func<T, object>> expression)
    {
      string propertyName = GetPropertyName(expression);
      return item.Properties[propertyName];
    }





    /// <summary>
    /// Sets the value of a given property using a strongly typed expression.
    /// </summary>
    public static void SetPropertyValue<T>(this ModelItem item, Expression<Func<T, object>> expression, object value)
    {
      string propertyName = GetPropertyName(expression);
      item.Properties[propertyName].SetValue(value);
    }


    /// <summary>
    /// Sets the value of a given property.
    /// </summary>
    public static void SetPropertyValue(this ModelItem item, string propertyName, object value)
    {
      item.Properties[propertyName].SetValue(value);
    }



    public static TVal GetPropertyValue<T, TVal>(this ModelItem item, Expression<Func<T, object>> expression, TVal defaultValue)
    {
      string propertyName = GetPropertyName(expression);
      return GetPropertyValue(item, propertyName, defaultValue);
    }


    public static TVal GetPropertyValue<TVal>(this ModelItem item, string propertyName, TVal defaultValue)
    {
      var property = item.Properties[propertyName];
      return property.IsSet ? (TVal)property.Value.GetCurrentValue() : defaultValue;
    }



    public static ModelItem GetPropertyValueItem<T>(this ModelItem item, Expression<Func<T, object>> expression)
    {
      string propertyName = GetPropertyName(expression);
      return item.Properties[propertyName].Value;
    }


    public static void AddAttributes<T>(this AttributeTableBuilder tableBuilder, params Attribute[] attributes)
    {
      Type type = typeof (T);
      tableBuilder.AddCustomAttributes(type, attributes);
    }
  }
}