﻿// Sketchables.NET - Design Time Support Libraries
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Gnu General Public License (GPL) v3
// Please see http://www.gnu.org/licenses/gpl.html for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Windows;
using Microsoft.Windows.Design.Interaction;
using Microsoft.Windows.Design.Metadata;
using Microsoft.Windows.Design.Model;

namespace Sketchables.Design
{
  public class ItemActionContextMenuProvider : PrimarySelectionContextMenuProvider
  {
    private readonly MenuGroup navigationGroup;
    private readonly MenuGroup decoratedActionsGroup;

		/// <summary>
		/// Type identifer for <c>System.Windows.Interactivity.Interaction</c>.
		/// </summary>
    private static readonly TypeIdentifier InteractionType = new TypeIdentifier("System.Windows.Interactivity.Interaction");

		/// <summary>
		/// Type identifier for Sketchable's behavior that allows the user
		/// to navigate to another screen.
		/// </summary>
    private static readonly TypeIdentifier NavigationBehaviorType = new TypeIdentifier("Sketchables.Behaviors.SketchItemNavigationBehavior");


		/// <summary>
		/// Type identifier for Sketchable's behavior that allows the user
		/// to invoke another action.
		/// </summary>
		private static readonly TypeIdentifier DecoratedItemActionBehaviorType = new TypeIdentifier("Sketchables.Behaviors.SketchableItemAction");



    public ItemActionContextMenuProvider()
    {
      UpdateItemStatus += OnUpdateItemStatus;

      navigationGroup = new MenuGroup("Sketchable_Navigation", "Sketchables: Navigate...") {HasDropDown = true};
      decoratedActionsGroup = new MenuGroup("Sketchable_Actions", "Sketchables: Action for...") {HasDropDown = true};

      Items.Add(navigationGroup);
      Items.Add(decoratedActionsGroup);
    }


		private void OnUpdateItemStatus(object sender, MenuActionEventArgs e)
		{
      navigationGroup.Items.Clear();
			decoratedActionsGroup.Items.Clear();
      
      // Get a ModelItem which represents the selected control. 
      ModelItem selectedControl = e.Selection.PrimarySelection;

      //determine all screens
      try
      {
        //determine all items
        dynamic model = selectedControl.GetCurrentValue();
        IEnumerable<dynamic> descriptions = model.ItemDescriptions;

        foreach (var sketchItemDescription in descriptions.OrderBy(d => d.Index))
        {
					//get the item's index
					int index = sketchItemDescription.Index;

					//create menu group with formatted name
          string itemName = sketchItemDescription.Name.Trim();
          if (itemName.Length > 20) itemName = itemName.Substring(0, 17) + "...";


					CreateDecoratedActionItem(index, itemName);
        	CreateNavigationMenuItem(index, itemName, selectedControl);
        }
      }
      catch (Exception exception)
      {
        MessageBox.Show(exception.ToString());
      }
    }

  	private void CreateDecoratedActionItem(int itemIndex, string itemName)
  	{
  		var action = new MenuAction(itemName);
			action.Execute += (s, args) => SetDecorationBehavior(args, itemIndex);
			decoratedActionsGroup.Items.Add(action);
  	}




  	/// <summary>
		/// Creates a sub item in the context menu that corresponds to a given sketch item. The
		/// menu item itself is a group that provides a list of screens that service as navigation
		/// targets.
		/// </summary>
		private void CreateNavigationMenuItem(int itemIndex, string itemName, ModelItem selectedControl)
		{
			//get the current behaviors
			var id = new PropertyIdentifier(InteractionType, "Behaviors");
			ModelProperty behaviorsProperty = selectedControl.Properties[id];
			ModelItemCollection behaviors = behaviorsProperty.Collection;

			//determine all screens
			IEnumerable<ScreenInfo> screens = GetScreens();

			string groupName = "from {0} to";
			groupName = String.Format(groupName, itemName);
			var itemGroup = new MenuGroup(groupName) { HasDropDown = true };

			foreach (ScreenInfo screen in screens)
			{
				ScreenInfo screenLocal = screen;

				bool isChecked = GetItemNavigationActions(behaviors, itemIndex, screen).Count() > 0;
				var action = new MenuAction(screen.DisplayName)
				             	{
				             		Checkable = true,
				             		Checked = isChecked
				             	};

				action.Execute += (s, args) => SetTriggerAction(args, screenLocal, itemIndex, isChecked);
				
				itemGroup.Items.Add(action);
			}

			navigationGroup.Items.Add(itemGroup);
		}



		/// <summary>
		/// Gets all currently defined navigation action behaviors of the selected control.
		/// </summary>
		/// <returns></returns>
    private static IEnumerable<ModelItem> GetItemNavigationActions(IEnumerable<ModelItem> behaviors, int? itemIndex,
                                                     ScreenInfo screenInfo = null)
    {
      return from b in behaviors
             where b.IsItemOfType(NavigationBehaviorType) &&
                   b.GetPropertyValue<int?>("ItemIndex", null) == itemIndex &&
                   (screenInfo == null || b.GetPropertyValue("TargetScreen", "") == screenInfo.ClassName)
             select b;
    }




		/// <summary>
		/// Creates a <c>SketchableItemAction</c>, which is a behavior that can be used
		/// to trigger a decorated action.
		/// </summary>
		private static void SetDecorationBehavior(MenuActionEventArgs args, int itemIndex)
		{
			try
			{
				ModelItem modelItem = args.Selection.PrimarySelection;

				var id = new PropertyIdentifier(InteractionType, "Behaviors");
				ModelProperty behaviorsProperty = modelItem.Properties[id];
				ModelItemCollection behaviors = behaviorsProperty.Collection;

				//create navigation action
				ModelItem navBehavior = ModelFactory.CreateItem(modelItem.Context, DecoratedItemActionBehaviorType,
																CreateOptions.InitializeDefaults);

				//this will cause conflicts for multiple actions
				string behaviorName = String.Format("ItemAction_For_{0:00}", itemIndex);
				behaviorName = UniqueName(behaviorName, behaviors);
				navBehavior.Name = behaviorName;

				navBehavior.SetPropertyValue("ItemIndex", itemIndex);

				//add behavior to collection
				behaviors.Add(navBehavior);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.ToString());
			}
		}



    /// <summary>
    /// Creates a navigation action if the user selects a given context menu
    /// entry.
    /// </summary>
    private static void SetTriggerAction(MenuActionEventArgs args, ScreenInfo screenInfo, int itemIndex, bool removeExisting)
    {
      try
      {
        ModelItem modelItem = args.Selection.PrimarySelection;
        
        var id = new PropertyIdentifier(InteractionType, "Behaviors");
        ModelProperty behaviorsProperty = modelItem.Properties[id];
        ModelItemCollection behaviors = behaviorsProperty.Collection;

        //check if we already have an existing navigation for that item. if yes, remove it
        //-> this also cleans up older invalid navigations in case the screens were renamed
        var obsoletes = GetItemNavigationActions(behaviors, itemIndex).ToList();
        obsoletes.ForEach(mi => behaviors.Remove(mi));
				
				//if the item was already checked, we only want to remove, so we're done
				if(removeExisting) return;

        //create navigation action
        ModelItem navBehavior = ModelFactory.CreateItem(modelItem.Context, NavigationBehaviorType,
                                CreateOptions.InitializeDefaults);

        string behaviorName = String.Format("Nav_{0:00}_{1}", itemIndex, screenInfo.DisplayName).Replace(" ", "_");
      	behaviorName = UniqueName(behaviorName, behaviors);
				navBehavior.Name = behaviorName; //<- this might still cause invalid names, but easy to fix...
        
        navBehavior.SetPropertyValue("ItemIndex", itemIndex);
        navBehavior.SetPropertyValue("TargetScreen", screenInfo.ClassName);

        //add behavior to collection
        behaviors.Add(navBehavior);
      }
      catch (Exception e)
      {
        MessageBox.Show(e.ToString());
      }

    }


		/// <summary>
		/// Guarantees a unique behavior name.
		/// </summary>
		private static string UniqueName(string suggestedName, IEnumerable<ModelItem> behaviors)
		{
			int counter = 0;
			string uniqueName = suggestedName;
			while (true)
			{
				//use local variable (modified closure)
				string name = uniqueName;
				bool isUsedName = behaviors.Any(b => b.Name == name);

				if (!isUsedName) break;
				uniqueName = String.Format("{0}.{1}", suggestedName, ++counter);
			}

			return uniqueName;
		}


		/// <summary>
		/// Determines all available screens at design time.
		/// </summary>
		/// <returns></returns>
    private static IEnumerable<ScreenInfo> GetScreens()
    {
      var list = new List<ScreenInfo>();

      const string runtimeAssembly = "Microsoft.Expression.Prototyping";
      var assemblies = AppDomain.CurrentDomain.GetAssemblies();
      var assembly = assemblies.FirstOrDefault(a => a.GetName().Name.Equals(runtimeAssembly));

      if (assembly == null) return list;

      const string typeName = "Microsoft.Expression.Prototyping.PrototypingContext";
      var type = assembly.GetType(typeName, true);
      PropertyInfo pi = type.GetProperty("ActiveDataModel", BindingFlags.Static | BindingFlags.Public);

      dynamic dataModel = pi.GetValue(null, new object[0]);
      if (dataModel == null) return list;

      //resolve Screens property through reflection again (dynamic fails, surprisingly)
      Type modelType = dataModel.GetType();
      pi = modelType.GetProperty("Screens");
      dynamic screens = pi.GetValue(dataModel, new object[0]);

      foreach (dynamic screen in screens)
      {
        list.Add(new ScreenInfo { ClassName = screen.ClassName, DisplayName = screen.DisplayName });
      }
      return list;
    }

		/// <summary>
		/// Wraps information about a given screen.
		/// </summary>
    class ScreenInfo
    {
      public string DisplayName { get; set; }

      public string ClassName { get; set; }
    }
  }
}
