﻿// Sketchables.NET - Design Time Support Libraries
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Gnu General Public License (GPL) v3
// Please see http://www.gnu.org/licenses/gpl.html for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using Microsoft.Windows.Design;
using Microsoft.Windows.Design.Features;
using Microsoft.Windows.Design.Metadata;
using Sketchables.Design.ContentEditor;
using Sketchables.Design.Ioc;


namespace Sketchables.Design
{
	internal abstract class MetadataStore : IProvideAttributeTable
	{
		/// <summary>
		/// Toolbox category of all Sketchables.
		/// </summary>
		public static readonly ToolboxCategoryAttribute ToolBoxCategory = new ToolboxCategoryAttribute("Sketchables");

		/// <summary>
		/// Resolves either SL or WPF types at runtime.
		/// </summary>
		public SketchableTypeFactory TypeFactory { get; private set; }


		protected MetadataStore(SketchableTypeFactory typeFactory)
		{
			TypeFactory = typeFactory;
			Container.Register(typeFactory);
		}


		/// <summary>
		/// Gets the attribute table provided by the class.
		/// </summary>
		/// <returns>
		/// The <see cref="T:Microsoft.Windows.Design.Metadata.AttributeTable"/> provided by the class.
		/// </returns>
		public AttributeTable AttributeTable
		{
			get
			{
				var tableBuilder = new AttributeTableBuilder();

				//register controls - there's support for Sketchables and a few built-in types
				RegisterSketchableControl(tableBuilder);
				RegisterTabControl(tableBuilder);
				RegisterListBoxControl(tableBuilder);

				return tableBuilder.CreateTable();
			}
		}


		private void RegisterSketchableControl(AttributeTableBuilder tableBuilder)
		{
			Type baseType = TypeFactory.GetPlatformType(SketchablePlatformTypes.SketchableControl);

			//Add the adorner provider to the design-time metadata.
			tableBuilder.AddCustomAttributes(
				baseType,
				new FeatureAttribute(typeof (InPlaceEditorProvider)));

			//register custom initializers
			tableBuilder.AddCustomAttributes(
				baseType,
				new FeatureAttribute(typeof (SketchableInitializer)));

			//register context menu
			RegisterItemActionContextMenu(tableBuilder);

			//put control into dedicated toolbox category
			tableBuilder.AddCustomAttributes(baseType, new Attribute[] {ToolBoxCategory});
		}


		/// <summary>
		/// Registers context menu support for default item actions.
		/// </summary>
		private void RegisterItemActionContextMenu(AttributeTableBuilder tableBuilder)
		{
			// Add the menu provider to the design-time metadata.
			var featureAttribute = new FeatureAttribute(typeof (ItemActionContextMenuProvider));

			tableBuilder.AddCustomAttributes(
				TypeFactory.GetPlatformType(SketchablePlatformTypes.BreadCrumb),
				featureAttribute);

			tableBuilder.AddCustomAttributes(
				TypeFactory.GetPlatformType(SketchablePlatformTypes.SketchableListBox),
				featureAttribute);

			tableBuilder.AddCustomAttributes(
				TypeFactory.GetPlatformType(SketchablePlatformTypes.DataGrid),
				featureAttribute);
		}


		/// <summary>
		/// Registers designer capabilities for TabControl.
		/// </summary>
		private void RegisterTabControl(AttributeTableBuilder tableBuilder)
		{
			Type baseType = TypeFactory.GetPlatformType(SketchablePlatformTypes.TabControl);

			//Add the adorner provider to the design-time metadata.
			tableBuilder.AddCustomAttributes(
				baseType,
				new FeatureAttribute(typeof (InPlaceEditorProvider)));

			//put control into dedicated toolbox category
			tableBuilder.AddCustomAttributes(baseType, new Attribute[] {ToolBoxCategory});
		}


		/// <summary>
		/// Registers designer capabilities for ListBox.
		/// </summary>
		private void RegisterListBoxControl(AttributeTableBuilder tableBuilder)
		{
			Type baseType = TypeFactory.GetPlatformType(SketchablePlatformTypes.ListBox);

			//Add the adorner provider to the design-time metadata.
			tableBuilder.AddCustomAttributes(
				baseType,
				new FeatureAttribute(typeof (InPlaceEditorProvider)));

			//put control into dedicated toolbox category
			tableBuilder.AddCustomAttributes(baseType, new Attribute[] {ToolBoxCategory});
		}
	}
}