﻿// Sketchables.NET - Design Time Support Libraries
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Gnu General Public License (GPL) v3
// Please see http://www.gnu.org/licenses/gpl.html for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE



using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using Microsoft.Windows.Design.Model;

namespace Sketchables.Design.Markup.Converters
{
  /// <summary>
  /// A converter that handles various types of <see cref="ItemsControl"/>.
  /// </summary>
  /// <typeparam name="TParent"></typeparam>
  /// <typeparam name="TItem"></typeparam>
  /// <typeparam name="TItemInfo"></typeparam>
  public abstract class ItemsControlConverter<TParent, TItemInfo> : ISketchingMarkupConverter
    where TParent:ItemsControl
    where TItemInfo:ItemInfo, new()
  {
    /// <summary>
    /// Gets sketching markup that is being displayed to the user.
    /// </summary>
    /// <param name="sketchedItem">The processed sketching object. This
    /// <see cref="ModelItem"/> represents either a framework type or
    /// a supported external type.</param>
    /// <returns>Sketching markup that represents the item.</returns>
    public virtual string GetMarkup(ModelItem sketchedItem)
    {
      ModelItemCollection itemsCollection = sketchedItem.GetProperty<TParent>(p => p.Items).Collection;

      var builder = new StringBuilder();

      string displayName = GetItemValueProperty();
      foreach (ModelItem childItem in itemsCollection)
      {
        string itemValue = childItem.GetPropertyValue(displayName, "");
        var item = new SketchItem { Value = itemValue };

        string enabledProperty = GetIsEnabledPropertyName();
        bool isEnabled = childItem.GetPropertyValue(enabledProperty, true);
        if (!isEnabled)
        {
          item.AddAttributeDirective(new Directive(Directive.Disabled));
        }

        builder.AppendLine(item.Value);
      }

      return builder.ToString();
    }



    /// <summary>
    /// Updates the underlying object with updated markup.
    /// </summary>
    /// <param name="parentModel">The processed sketching object. This
    /// <see cref="ModelItem"/> represents either a framework type or
    /// a supported external type.</param>
    /// <param name="newMarkup">The updated markup which should be applied.</param>
    public virtual void UpdateSketchObject(ModelItem parentModel, string newMarkup)
    {
      //get items and clear the parent
      ModelItemCollection itemsCollection = parentModel.GetProperty<TParent>(p => p.Items).Collection;

      //get a list of currently mantained items of the control
      var childEntries = itemsCollection.Select((m, i) => new TItemInfo { Model = m, Index = i }).ToList();
      
      //split the new markup and reset indices (in case we had null items in between)
      IEnumerable<SketchItem> sketchItems = newMarkup.SplitMarkupString().Where(i => i != null).ToArray();
      sketchItems.ResetIndices();

      PrepareParentItemForUpdate(parentModel, itemsCollection, sketchItems);

      foreach (SketchItem sketchItem in sketchItems)
      {
        //check whether we already have that child item
        SketchItem si = sketchItem; //prevent modified closure
        var existingEntryInfo = childEntries.FirstOrDefault(t => t.Matches(si) && !t.IsUsed);

        //either reuse an existing item, or create a new one
        ModelItem currentItem;
        if (existingEntryInfo != null)
        {
          currentItem = existingEntryInfo.Model;
          existingEntryInfo.IsUsed = true;
          //cache the designated index
          existingEntryInfo.Index = sketchItem.Index;
        }
        else
        {
          //create tab item
          currentItem = CreateChildItem(sketchItem, parentModel);

          //store in both the child collection and the info collection
          itemsCollection.Add(currentItem);
          childEntries.Add(new TItemInfo { Model = currentItem, IsUsed = true, Index = sketchItem.Index });
        }


        //this may apply to both new and existing items
        string enabledProperty = GetIsEnabledPropertyName();
        if (!sketchItem.IsEnabled())
        {
          currentItem.SetPropertyValue(enabledProperty, false);
        }
        else
        {
          currentItem.Properties[enabledProperty].ClearValue();
        }
      }


      //remove all items that are no longer in use
      for (int i = childEntries.Count - 1; i >= 0; i--)
      {
        if (!childEntries[i].IsUsed)
        {
          childEntries.RemoveAt(i);
          itemsCollection.RemoveAt(i);
        }
      }

      //reset order
      foreach (var childInfo in childEntries)
      {
        var index = itemsCollection.IndexOf(childInfo.Model);
        itemsCollection.Move(index, childInfo.Index);
      }

      //allow final customization
      CustomizeParentItem(parentModel, itemsCollection, sketchItems);
    }


    /// <summary>
    /// Can be overridden in order to do some customization prior to parsing.
    /// </summary>
    protected virtual void PrepareParentItemForUpdate(ModelItem parentModel, ModelItemCollection itemsCollection, IEnumerable<SketchItem> sketchItems)
    {
    }


    /// <summary>
    /// Can be overridden in order to customize the parent model item if necessary.
    /// </summary>
    protected virtual void CustomizeParentItem(ModelItem parentModel, ModelItemCollection itemsCollection, IEnumerable<SketchItem> sketchItems)
    {
    }


    /// <summary>
    /// Creates a single item for the parent.
    /// </summary>
    private ModelItem CreateChildItem(SketchItem sketch, ModelItem parentModel)
    {
      //recreate blank tab item
      ModelItem childItem = ModelFactory.CreateItem(parentModel.Context, GetChildItemType(), CreateOptions.None);

      //set main property
      childItem.SetPropertyValue(GetItemValueProperty(), sketch.ContentMarkup);

      //style resource
      string resourceDirective = UseDynamicResource ? "DynamicResource" : "StaticResource";
      string styleName = GetItemStyleName();
      string style = String.Format("{0} {1}", resourceDirective, styleName);
      style = "{" + style + "}";
      childItem.SetPropertyValue("Style", style);

      //Give the implementation the chance to customize the default
      CustomizeItemDefaults(childItem);

      return childItem;
    }


    /// <summary>
    /// Gets the type of the contained items, e.g. a <c>TabItem</c>
    /// for a <c>TabControl</c>.
    /// </summary>
    /// <returns></returns>
    protected abstract Type GetChildItemType();

    /// <summary>
    /// Returns the name of the property that is used to
    /// identify the item. This is also the identifier that
    /// is being used to produce sketching markup.
    /// </summary>
    protected abstract string GetItemValueProperty();

    /// <summary>
    /// Gets the name of the property that indicates whether
    /// the item is enabled or not.
    /// </summary>
    protected abstract string GetIsEnabledPropertyName();

    
    /// <summary>
    /// Gets an item that represents a tab's default content. This item will
    /// be assigned to it's <c>Content</c> property.
    /// </summary>
    /// <returns>Tab content model item.</returns>
    protected abstract void CustomizeItemDefaults(ModelItem item);

    /// <summary>
    /// Generates a resource string for the 
    /// </summary>
    /// <returns></returns>
    protected abstract string GetItemStyleName();

    /// <summary>
    /// Whether to use a dynamic or static resource.
    /// </summary>
    protected abstract bool UseDynamicResource { get; }
  }
}
