﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;


namespace Sketchables
{
  /// <summary>
  /// Represents a directive with an optional value.
  /// </summary>
  public class Directive
  {
    /// <summary>
    /// The prefix character of an attribute directives section.
    /// </summary>
    public const string AttributeDirectivesPrefix = "{";

    /// <summary>
    /// The suffix character of the default directives
    /// sesion.
    /// </summary>
    public const string AttributeDirectivesSuffix = "}";


    /// <summary>
    /// The prefix character of a format directive section.
    /// </summary>
    public const string FormatDirectivePrefix = "[";

    /// <summary>
    /// The suffix character of an format directive section.
    /// </summary>
    public const string FormatDirectiveSuffix = "]";



    /// <summary>
    /// Sets the width of the decorated item. The value may
    /// be an int value or another string (e.g. a grid length)
    /// depending on the context.
    /// </summary>
    public const string Width = "w";

    /// <summary>
    /// Selects / activates the decorated item.
    /// </summary>
    public const string Selected = "x";

    /// <summary>
    /// Disables the decorated item.
    /// </summary>
    public const string Disabled = "-";

    /// <summary>
    /// Indicates the item should be treated as a label.
    /// </summary>
    public const string Label = "t";

    /// <summary>
    /// The string used to separate directive keys and values.
    /// </summary>
    public const string ValueSeparator = ":";


    /// <summary>
    /// The name (key) of the directive.
    /// </summary>
    public string Name { get; set; }

    /// <summary>
    /// An optional value. Values and directive names are separated
    /// by a colon, e.g. <c>w:123</c>.
    /// </summary>
    public string Value { get; set; }


    /// <summary>
    /// Creates an empty directive.
    /// </summary>
    public Directive()
    {
      //init defaults
      Name = Value = String.Empty;
    }

    /// <summary>
    /// Creates the directive with a string that represents
    /// the directive. If the directive contains a colon
    /// separator (':'), the submitted string is splitted into
    /// a name/value pair. Otherwise, the full string is assigned
    /// to the <see cref="Name"/> property.
    /// </summary>
    /// <param name="directiveString">A directive.</param>
    public Directive(string directiveString)
    {
      //make sure value is set
      Value = String.Empty;

      if (directiveString == null) directiveString = "";

      int separatorIndex = directiveString.IndexOf(ValueSeparator);
      if(separatorIndex == -1)
      {
        Name = directiveString;
      }
      else
      {
        Name = directiveString.Substring(0, separatorIndex);
        if(separatorIndex < directiveString.Length)
        {
          Value = directiveString.Substring(separatorIndex + 1)
                                 .TrimSingleSpace()
                                 .Unescape();
        }
      }
    }



    /// <summary>
    /// Returns the name (and the value, if available).
    /// </summary>
    public override string ToString()
    {
      if (String.IsNullOrEmpty(Value)) return Name;

      return String.Format("{0}{1}{2}", Name, ValueSeparator, Value);
    }
  }
}