﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Collections.Generic;
using System.Linq;

namespace Sketchables
{
  public static class ItemExtensions
  {
    /// <summary>
    /// The item is checked or selected. Selection directive
    /// is <c>[x]</c>.
    /// </summary>
    public static bool IsSelected(this SketchItem item)
    {
      return item.HasDirective(Directive.Selected);
    }


    /// <summary>
    /// The item is disabled. Disabled directive
    /// is <c>[-]</c>.
    /// </summary>
    public static bool IsEnabled(this SketchItem item)
    {
      return !item.HasDirective(Directive.Disabled);
    }


    /// <summary>
    /// The item is a label that shouldn't be treated like regular
    /// items. Label directive is <c>[t]</c>.
    /// </summary>
    public static bool IsLabel(this SketchItem item)
    {
      return item.HasDirective(Directive.Label);
    }


    /// <summary>
    /// Checks whether the item's <see cref="SketchItem.ContentMarkup"/>
    /// is a formatting directive, which is a string that is surrounded
    /// by curly braces, e.g. <c>{@:http://www.google.com}</c>.
    /// </summary>
    /// <returns></returns>
    public static bool IsFormattingDirective(this SketchItem item)
    {
      if (String.IsNullOrWhiteSpace(item.ContentMarkup)) return false;
      var content = item.ContentMarkup.Trim();
      return content.StartsWith(Directive.FormatDirectivePrefix) && content.EndsWith(Directive.FormatDirectiveSuffix);
    }


    /// <summary>
    /// Resets indices in a list of items.
    /// </summary>
    /// <param name="items"></param>
    public static void ResetIndices(this IEnumerable<SketchItem> items)
    {
      int index = 0;
      foreach (var sketchItem in items)
      {
        if (sketchItem != null) sketchItem.Index = index;
        index++;
      }
    }


    /// <summary>
    /// Updates <see cref="SketchItem.Value"/> and <see cref="SketchItem.Directives"/>
    /// of the item with an attribute directive.
    /// </summary>
    public static void AddAttributeDirective(this SketchItem item, Directive directive)
    {
      if(!item.Value.Trim().StartsWith(Directive.AttributeDirectivesPrefix))
      {
        //add a directive in front
        item.Value = String.Format("{0}{1}{2}", Directive.AttributeDirectivesPrefix, Directive.AttributeDirectivesSuffix, item.Value);        
      }

      //insert directive
      int directivesEnd = item.Value.IndexOf(Directive.AttributeDirectivesSuffix);
      item.Value = item.Value.Insert(directivesEnd, directive.ToString());

      //replace directives collection
      var directives = item.Directives.ToList();
      directives.Add(directive);
      item.Directives = directives;
    }
  }
}
