using System.Windows;
using Microsoft.Expression.Prototyping.Behavior;
using Sketchables.Controls;

namespace Sketchables.Behaviors
{
  public class SketchItemNavigationBehavior : TriggerActionDecoratorBehavior<SketchableControl, NavigateToScreenAction>
  {
    private readonly NavigateToScreenAction navigationAction;

    #region ItemIndex

    public static readonly DependencyProperty ItemIndexProperty =
      DependencyProperty.Register("ItemIndex", typeof(int?), typeof(SketchItemNavigationBehavior),
                                  new PropertyMetadata(null));

    /// <summary>
    /// The item that is attached to this event.
    /// </summary>
    public int? ItemIndex
    {
      get { return (int?)GetValue(ItemIndexProperty); }
      set { SetValue(ItemIndexProperty, value); }
    }

    #endregion


    #region TargetScreen

    public static readonly DependencyProperty TargetScreenProperty =
        DependencyProperty.Register("TargetScreen", typeof(string), typeof(SketchItemNavigationBehavior),
            new PropertyMetadata("", OnTargetScreenChanged));


    private static void OnTargetScreenChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      ((SketchItemNavigationBehavior) d).navigationAction.TargetScreen = (string)e.NewValue;
    }

    /// <summary>
    /// The navigation target screen.
    /// </summary>
    public string TargetScreen
    {
      get { return (string)GetValue(TargetScreenProperty); }
      set { SetValue(TargetScreenProperty, value); }
    }

    #endregion


    public SketchItemNavigationBehavior()
    {
      navigationAction = new NavigateToScreenAction();
      SetAction(navigationAction);
    }


    protected override void OnAttached()
    {
      base.OnAttached();

      var selectable = AssociatedObject as ISelectableSketchControl;
      if (selectable != null)
      {
        selectable.SketchItemChanged += OnItemChanged;
      }
    }


    protected override void OnDetaching()
    {
      var selectable = AssociatedObject as ISelectableSketchControl;
      if (selectable != null)
      {
        selectable.SketchItemChanged -= OnItemChanged;
      }

      base.OnDetaching();
    }


    /// <summary>
    /// Event handler for selection changes. Triggers the underlying
    /// navigation action, if the <see cref="ItemIndex"/> matches
    /// the selected item of the associated object.
    /// </summary>
    private void OnItemChanged(object sender, SketchItemEventArgs e)
    {
      if (ItemIndex == null || e.Item.Index == ItemIndex)
      {
        InvokeDecorated(e.Item);
      }
    }

  }
}
