using System.Windows;
using Microsoft.Expression.Prototyping.Behavior;
using Sketchables.Controls;
using TriggerAction = System.Windows.Interactivity.TriggerAction;

namespace Sketchables.Behaviors
{
  /// <summary>
  /// A behavior that triggers an arbitrary decorated action if the underlying <see cref="SketchableControl"/>
  /// fires a selection change event for a given <see cref="SketchItem"/>.
  /// </summary>
  public class SketchableItemAction : TriggerActionDecoratorBehavior<SketchableControl, TriggerAction>
  {

    #region Action

    /// <summary>
    /// Action Dependency Property
    /// </summary>
    public static readonly DependencyProperty ActionProperty =
        DependencyProperty.Register("Action", typeof(TriggerAction), typeof(SketchableItemAction),
            new PropertyMetadata(null, ActionPropertyChanged));


    /// <summary>
    /// Gets or sets the underlying (decorated) <see cref="TriggerAction"/>.
    /// </summary>
    public TriggerAction Action
    {
      get { return (TriggerAction)GetValue(ActionProperty); }
      set { SetValue(ActionProperty, value); }
    }


    /// <summary>
    /// A static callback listener which is being invoked if the
    /// <see cref="ActionProperty"/> dependency property has
    /// been changed.
    /// </summary>
    /// <param name="d">The currently processed owner of the property.</param>
    /// <param name="e">Provides information about the updated property.</param>
    private static void ActionPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var owner = (SketchableItemAction)d;

      var oldValue = e.OldValue;
      var newValue = (TriggerAction)e.NewValue;

      if (ReferenceEquals(oldValue, newValue)) return;

      owner.SetAction(newValue);
    }

    #endregion


    #region ItemIndex

    public static readonly DependencyProperty ItemIndexProperty =
      DependencyProperty.Register("ItemIndex", typeof(int?), typeof(SketchableItemAction),
                                  new PropertyMetadata(null));

    /// <summary>
    /// The item that is attached to this event.
    /// </summary>
    public int? ItemIndex
    {
      get { return (int?)GetValue(ItemIndexProperty); }
      set { SetValue(ItemIndexProperty, value); }
    }

    #endregion


    /// <summary>
    /// Creates the behavior with a default action.
    /// </summary>
    public SketchableItemAction() : base(new NavigateToScreenAction())
    {
    }


    protected override void OnAttached()
    {
      base.OnAttached();

      var selectable = AssociatedObject as ISelectableSketchControl;
      if (selectable != null)
      {
        selectable.SketchItemChanged += OnItemChanged;
      }
    }


    protected override void OnDetaching()
    {
      var selectable = AssociatedObject as ISelectableSketchControl;
      if (selectable != null)
      {
        selectable.SketchItemChanged -= OnItemChanged;
      }

      base.OnDetaching();
    }


    /// <summary>
    /// Event handler for selection changes. Triggers the underlying
    /// navigation action, if the <see cref="ItemIndex"/> matches
    /// the selected item of the associated object.
    /// </summary>
    private void OnItemChanged(object sender, SketchItemEventArgs e)
    {
      if (ItemIndex == null || e.Item.Index == ItemIndex)
      {
        InvokeDecorated(e.Item);
      }
    }

  }
}
