﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using TriggerAction = System.Windows.Interactivity.TriggerAction;


namespace Sketchables.Behaviors
{
  /// <summary>
  /// Simple decorator class that allows a custom trigger to wrap
  /// an existing <see cref="System.Windows.Interactivity.TriggerAction"/>
  /// that cannot be extended.
  /// </summary>
  public abstract class TriggerActionDecoratorBehavior<T, TAction> : Behavior<T> where T : DependencyObject
    where TAction : TriggerAction
  {

    /// <summary>
    /// A dummy trigger that is used to invoke the decorated <see cref="Action"/>.
    /// </summary>
    private readonly SimpleTrigger<T> trigger = new SimpleTrigger<T>();


    protected TriggerActionDecoratorBehavior()
    {
    }

    protected TriggerActionDecoratorBehavior(TAction decoratedAction)
    {
      SetAction(decoratedAction);
    }


    protected void SetAction(TAction action)
    {
      if(trigger.Actions.Count > 0)
      {
        trigger.Actions.Clear();
      }

			if(action != null) trigger.Actions.Add(action);
    }


    /// <summary>
    /// Invokes the action.
    /// </summary>
    /// <param name="parameter">The parameter to the action. If the action
    /// does not require a parameter, the parameter may be set to a null reference.</param>
    protected void InvokeDecorated(object parameter)
    {
      trigger.Invoke(parameter);
    }


    /// <summary>
    /// Called after the action is attached to an AssociatedObject.
    /// </summary>
    protected override void OnAttached()
    {
      trigger.Attach(AssociatedObject);
    }


    /// <summary>
    /// Called when the action is being detached from its AssociatedObject,
    /// but before it has actually occurred.
    /// </summary>
    protected override void OnDetaching()
    {
      trigger.Detach();
      base.OnDetaching();
    }


    /// <summary>
    /// A dummy trigger that is used to invoke the decorated trigger action.
    /// </summary>
    protected class SimpleTrigger<TObj> : TriggerBase<TObj> where TObj : DependencyObject
    {
      public void Invoke(object parameter)
      {
        InvokeActions(parameter);
      }
    }
  }




}
