﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.Expression.Prototyping.SketchControls;

namespace Sketchables.Controls
{
  public static partial class ControlBuilder
  {
    /// <summary>
    /// Creates a <see cref="HyperlinkButton"/> with the submitted text.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="linkText">Displayed link.</param>
    /// <returns>Built control.</returns>
    public static FrameworkElement CreateHyperlink(this SketchItem item, string linkText)
    {
      if (item.IsSelected())
      {
        //render selected breadcrumbs as normal text
        var textBlock = CreateTextBlock(linkText);
        textBlock.Foreground = GetActiveHyperlinkBrush();
        return textBlock;
      }


      var button = new HyperlinkButton
                     {
                       Content = linkText,
                       IsEnabled = item.IsEnabled(),
                       Tag = item
                     }.WithStyle(SketchStyleNames.HyperlinkButton);

      return button;
    }


    /// <summary>
    /// Creates a rectangle with a sketched look and the specified stroke width.
    /// </summary>
    /// <param name="strokeWidth">The width of the sketched rectangle stroke. Defaults
    /// to 1.</param>
    public static FrameworkElement CreateSketchRectangle(double strokeWidth = 1)
    {
      return new SketchRectangleSL
               {
                 StrokeWidth = strokeWidth,
                 BorderBrush = new SolidColorBrush(Colors.Black)
               };
    }

    /// <summary>
    /// Simple parser function that converts a string into a <see cref="GridLength"/>.
    /// Apparently, we don't have a converter in SL.
    /// </summary>
    /// <param name="lengthString"></param>
    /// <returns>Converted value. If the value cannot be converted, a value of
    /// <see cref="GridLength.Auto"/> is returned.</returns>
    public static GridLength ParseGridLength(string lengthString)
    {
      var culture = CultureInfo.InvariantCulture;
      lengthString = (lengthString ?? "auto").Trim().ToLower(culture);

      if (lengthString == "auto")
      {
        return GridLength.Auto;
      }

      GridUnitType unitType;
      string numberString = "";
      if (lengthString.EndsWith("*"))
      {
        if (lengthString.Length == 1)
        {
          return new GridLength(1.0, GridUnitType.Star);
        }

        numberString = lengthString.Substring(0, lengthString.Length - 1);
        unitType = GridUnitType.Star;
      }
      else
      {
        numberString = lengthString;
        unitType = GridUnitType.Pixel;
      }

      double result;
      bool status = double.TryParse(numberString, NumberStyles.Number, culture, out result);
      return status ? new GridLength(result, unitType) : GridLength.Auto;
    }
  }
}