﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;


namespace Sketchables.Controls
{
  public static partial class ControlBuilder
  {
    public const string CheckBoxFormatDirective = "x";
    public const string RadioButtonFormatDirective = "o";
    public const string RatingFormatDirective = "*";
    public const string ProgressBarFormatDirective = "p";
    public const string HyperlinkFormatDirective = "@";
    public const string TextBoxFormatDirective = "txt";
    public const string ComboBoxFormatDirective = "cbo";
    public const string ButtonFormatDirective = "btn";


    private static readonly Random Randomizer = new Random();


    /// <summary>
    /// Creates a simple sketch control based on an item which either
    /// contains pure text or a formatting directive.
    /// </summary>
    /// <param name="item"></param>
    /// <returns></returns>
    public static FrameworkElement CreateFormattedSketchControl(this SketchItem item)
    {
      if(!item.IsFormattingDirective())
      {
        return CreateTextBlock(item.ContentMarkup);
      }


      string markup = item.ContentMarkup.Trim();
      markup = markup.Substring(1, markup.Length - 2);
      var directive = new Directive(markup);

      FrameworkElement element;

      switch(directive.Name)
      {
        case TextBoxFormatDirective:
          element = item.CreateTextBox(directive.Value);
          break;
        case ComboBoxFormatDirective:
          element = item.CreateComboBox(directive.Value);
          break;
        case CheckBoxFormatDirective:
          element = item.CreateCheckBox(directive.Value);
          break;
        case RadioButtonFormatDirective:
          element = item.CreateRadioButton(directive.Value);
          break;
        case ButtonFormatDirective:
          element = item.CreateButton(directive.Value);
          break;
        case RatingFormatDirective:
          int rating;
          bool ratingResult = int.TryParse(directive.Value, NumberStyles.Any, CultureInfo.InvariantCulture, out rating);
          element = item.CreateRating(ratingResult ? rating : (int?)null);
          break;
        case ProgressBarFormatDirective:
          double value;
          bool status = double.TryParse(directive.Value, NumberStyles.Any, CultureInfo.InvariantCulture, out value);
          element = item.CreateProgressBar(status ? value : (double?)null);
          break;
        case HyperlinkFormatDirective:
          element = item.CreateHyperlink(directive.Value);
          break;
        default:
          //unknown directive - just print out the pure value
          element = CreateTextBlock(directive.Value);
          break;
      }

      
      //set the width
      element.SetWidth(item);
      return element;
    }


    /// <summary>
    /// Returns a styled text element with the submitted text.
    /// </summary>
    public static TextBlock CreateTextBlock(string text)
    {
      return CreateTextBlock(text, LabelStyle.Normal);
    }


    /// <summary>
    /// Returns a styled text element with the submitted text.
    /// </summary>
    public static TextBlock CreateTextBlock(string text, LabelStyle style)
    {
      string styleName;
      switch (style)
      {
        case LabelStyle.Normal:
          styleName = SketchStyleNames.TextBlock;
          break;
        case LabelStyle.Title:
          styleName = SketchStyleNames.TextBlockTitle;
          break;
        case LabelStyle.Subtitle:
          styleName = SketchStyleNames.TextBlockSubtitle;
          break;
        default:
          styleName = SketchStyleNames.TextBlock;
          break;
      }

      var tb = new TextBlock().WithStyle(styleName);
      if (text != null) tb.Text = text;
      return tb;
    }



    /// <summary>
    /// Creates a styled <see cref="TextBox"/> control with the
    /// submitted text assigned.
    /// </summary>
    /// <param name="text">Text to be assigned to the
    /// control.</param>
    public static TextBox CreateTextBox(string text)
    {
      return new TextBox { Text = text ?? ""}.WithStyle(SketchStyleNames.TextBox);
    }


    /// <summary>
    /// Creates a <see cref="TextBox"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <returns>Built CheckBox control.</returns>
    public static TextBox CreateTextBox(this SketchItem item)
    {
      return CreateTextBox(item, item.ContentMarkup);
    }

    /// <summary>
    /// Creates a <see cref="TextBox"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="text">Text to be assigned to the
    /// control.</param>
    /// <returns>Built control.</returns>
    public static TextBox CreateTextBox(this SketchItem item, string text)
    {
      var tb = CreateTextBox(text);
      tb.IsEnabled = item.IsEnabled();
      return tb;
    }


    /// <summary>
    /// Creates a <see cref="Button"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <returns>Built control.</returns>
    public static Button CreateButton(this SketchItem item)
    {
      return CreateButton(item, item.ContentMarkup);
    }


    /// <summary>
    /// Creates a <see cref="Button"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="text">Text to be assigned to the
    /// control.</param>
    /// <returns>Built control.</returns>
    public static Button CreateButton(this SketchItem item, string text)
    {
      var btn = new Button().WithStyle(SketchStyleNames.Button);
      btn.Content = text;
      btn.FontSize = 16;
      btn.IsEnabled = item.IsEnabled();
      return btn;
    }




    /// <summary>
    /// Creates a <see cref="CheckBox"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <returns>Built CheckBox control.</returns>
    public static CheckBox CreateCheckBox(this SketchItem item)
    {
      return CreateCheckBox(item, item.ContentMarkup); 
    }

    /// <summary>
    /// Creates a <see cref="CheckBox"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="content">Content to be assigned to the
    /// control.</param>
    /// <returns>Built control.</returns>
    public static CheckBox CreateCheckBox(this SketchItem item, object content)
    {
      var cb = new CheckBox().WithStyle(SketchStyleNames.CheckBox);
      cb.Content = content;
      cb.IsChecked = item.IsSelected();
      cb.IsEnabled = item.IsEnabled();
      cb.VerticalAlignment = VerticalAlignment.Center;
      cb.Margin = new Thickness(2, 0, 0, 0);
      return cb;
    }


    /// <summary>
    /// Creates a <see cref="RadioButton"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <returns>Built control.</returns>
    public static RadioButton CreateRadioButton(this SketchItem item)
    {
      return CreateRadioButton(item, item.ContentMarkup);
    }

    /// <summary>
    /// Creates a <see cref="RadioButton"/> control with basic
    /// formatting.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="content">Content to be assigned to the
    /// control.</param>
    /// <returns>Built control.</returns>
    public static RadioButton CreateRadioButton(this SketchItem item, object content)
    {
      var rb = new RadioButton().WithStyle(SketchStyleNames.RadioButton);
      
      rb.Content = content;
      rb.IsChecked = item.IsSelected();
      rb.IsEnabled = item.IsEnabled();

      rb.VerticalAlignment = VerticalAlignment.Center;
      rb.Margin = new Thickness(2, 0, 0, 0);
      return rb;
    }


    /// <summary>
    /// Creates a <see cref="ProgressBar"/> control with an optionally fixed value
    /// or a random value assigned to it.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <param name="value">Progress bar value.</param>
    /// <returns>Built control.</returns>
    public static ProgressBar CreateProgressBar(this SketchItem item, double? value = null)
    {
      var pb = new ProgressBar().WithStyle(SketchStyleNames.ProgressBar);
      pb.IsEnabled = item.IsEnabled();
      pb.Value = (value ?? Randomizer.NextDouble())*100;
      pb.Margin = new Thickness(0, 2, 2, 0);
      return pb;
    }


    /// <summary>
    /// Creates a <see cref="ComboBox"/> control.
    /// </summary>
    /// <param name="item"></param>
    /// <param name="itemsString">A string that provides the combobox items,
    /// separated by commas</param>
    /// <returns></returns>
    public static ComboBox CreateComboBox(this SketchItem item, string itemsString)
    {
      var cb = new ComboBox().WithStyle(SketchStyleNames.ComboBox);
      var items = itemsString.SplitString().Select(s => new ComboBoxItem {Content = s})
        .ToArray();

      var selectedItem = items.FirstOrDefault(i => ((string) i.Content).IsEnclosedIn("(", ")"));
      if (selectedItem != null)
      {
        var content = selectedItem.Content.ToString().Trim();
        selectedItem.Content = content.Substring(1, content.Length - 2);
        selectedItem.IsSelected = true;
      }

      cb.SelectedItem = selectedItem;
      cb.ItemsSource = items;
      cb.FontSize = 16;
      cb.MinHeight = 16;
      return cb;
    }


    /// <summary>
    /// Creates a <see cref="Hyperlink"/> with the submitted text,
    /// wrapped into a <see cref="TextBlock"/>.
    /// </summary>
    /// <param name="item">The item that represents the control
    /// to be built.</param>
    /// <returns>Built control.</returns>
    public static FrameworkElement CreateHyperlink(this SketchItem item)
    {
      return CreateHyperlink(item, item.ContentMarkup);
    }


    public static FrameworkElement CreateRating(this SketchItem item, int? rating = null)
    {
      int stars = rating ?? Randomizer.Next(1, 6);
      stars = Math.Min(5, stars);
      stars = Math.Max(1, stars);
      var panel = new StackPanel {Orientation = Orientation.Horizontal};
      for (int i = 0; i < stars; i++)
      {
        panel.Children.Add(new StarSketch());
      }

      return panel;
    }


    /// <summary>
    /// Creates a sketch border and assigns the submitted element
    /// as its child.
    /// </summary>
    /// <param name="child">The child element that is supposed to be
    /// placed into the border.</param>
    /// <param name="strokeWidth">The width of the sketched rectangle stroke. Defaults
    /// to 1.</param>
    /// <returns>A border-shaped control that contains the submitted child element.</returns>
    public static FrameworkElement PutIntoBorder(this FrameworkElement child, double strokeWidth = 1)
    {
      var rectangle = CreateSketchRectangle(strokeWidth);

      var grid = new Grid();
      grid.Children.Add(rectangle);
      grid.Children.Add(child);
      return grid;
    }


    public static Brush GetHyperlinkBrush()
    {
      return new SolidColorBrush(Color.FromArgb(255, 115, 169, 216)); //#FF73A9D8
    }

    public static Brush GetActiveHyperlinkBrush()
    {
      return new SolidColorBrush(Color.FromArgb(255, 132, 213, 92)); //#FF84D55C
    }



    /// <summary>
    /// Checks whether the submitted <see cref="SketchItem"/> has a
    /// directive that specifieds the width, and assigns the
    /// parsed value to the <paramref name="element"/>.
    /// </summary>
    public static void SetWidth(this FrameworkElement element, SketchItem item)
    {
      var directive = item.TryGetDirective("w");
      if (directive == null) return;

      int width;
      bool status = int.TryParse(directive.Value, out width);
      if (status) element.Width = width;
    }
  }
}
