﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.Expression.Prototyping.SketchControls;

namespace Sketchables.Controls
{
  /// <summary>
  /// Renders a matrix of controls.
  /// </summary>
  public class SketchableTable : RuntimeCompositionSketchControl
  {
    #region RowHeight

    /// <summary>
    /// RowHeight Dependency Property
    /// </summary>
    public static readonly DependencyProperty RowHeightProperty =
        DependencyProperty.Register("RowHeight", typeof(GridLength), typeof(SketchableTable),
            new PropertyMetadata(new GridLength(1, GridUnitType.Star), OnRowHeightChanged));

    /// <summary>
    /// Gets or sets the RowHeight property.
    /// </summary>
    public GridLength RowHeight
    {
      get { return (GridLength)GetValue(RowHeightProperty); }
      set { SetValue(RowHeightProperty, value); }
    }

    /// <summary>
    /// Handles changes to the RowHeight property.
    /// </summary>
    private static void OnRowHeightChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var target = (SketchableTable)d;
      var oldRowHeight = (GridLength)e.OldValue;
      GridLength newRowHeight = target.RowHeight;
      target.OnRowHeightChanged(oldRowHeight, newRowHeight);
    }

    /// <summary>
    /// Provides derived classes an opportunity to handle changes to the RowHeight property.
    /// </summary>
    protected virtual void OnRowHeightChanged(GridLength oldRowHeight, GridLength newRowHeight)
    {
      RefreshRuntimeElement();
    }

    #endregion

    #region ShowTableLines

    /// <summary>
    /// ShowTableLines Dependency Property
    /// </summary>
    public static readonly DependencyProperty ShowTableLinesProperty =
        DependencyProperty.Register("ShowTableLines", typeof(bool), typeof(SketchableTable),
            new PropertyMetadata(false, OnShowTableLinesChanged));

    /// <summary>
    /// Gets or sets the ShowTableLines property. This dependency property 
    /// indicates whether the table lines should be visible or not.
    /// </summary>
    public bool ShowTableLines
    {
      get { return (bool)GetValue(ShowTableLinesProperty); }
      set { SetValue(ShowTableLinesProperty, value); }
    }

    /// <summary>
    /// Handles changes to the ShowTableLines property.
    /// </summary>
    private static void OnShowTableLinesChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var target = (SketchableTable)d;
      var oldShowTableLines = (bool)e.OldValue;
      bool newShowTableLines = target.ShowTableLines;
      target.OnShowTableLinesChanged(oldShowTableLines, newShowTableLines);
    }

    /// <summary>
    /// Provides derived classes an opportunity to handle changes to the ShowTableLines property.
    /// </summary>
    protected virtual void OnShowTableLinesChanged(bool oldShowTableLines, bool newShowTableLines)
    {
      RefreshRuntimeElement();
    }

    #endregion

    
    /// <summary>
    /// This abstract builder method is responsible for creating the
    /// <see cref="SketchableControl.RuntimeElement"/> that is displayed
    /// at runtime and represents the sketched item. This method is being
    /// invoked by the <see cref="RuntimeCompositionSketchControl.RefreshRuntimeElement"/> method. Do not invoke
    /// it in order to have the UI updated.
    /// </summary>
    /// <returns></returns>
    protected override UIElement CreateRuntimeElementInternal()
    {
      string content = (Content ?? "").TrimStart();

      //generate rows
      var rows = content.Replace("\r\n", "\n")
        .Split('\n')
        .Select(RowSketchItem.Create)
        .ToList();

      if (rows.Count == 0)
      {
        return null;
      }

      //prepare grid - use auto sized cells if there are col definitions missing in the header row
      //leave row heights implicitly star sized
      var columnCount = rows.Max(ri => ri == null ? 0 : ri.Cells.Count);
      if (columnCount == 0) return null;

      //the first row provides the column definitions
      RowSketchItem headerRow = rows[0];
      rows.RemoveAt(0);

      var grid = ContentUtil.PrepareGrid(columnCount, rows.Count);
      foreach (var colItem in headerRow.Cells)
      {
        //the first row is supposed to provide header information only
        var width = ControlBuilder.ParseGridLength(colItem.Value);
        grid.ColumnDefinitions[colItem.Index].Width = width;
      }
      
      foreach (var rowItem in rows)
      {
        int rowIndex = rowItem.Index - 1;

        grid.RowDefinitions[rowIndex].Height = RowHeight;

        //skip empty cells
        var cells = rowItem.Cells.Where(c => !string.IsNullOrEmpty(c.Value)).ToArray();
        foreach (var cellItem in cells)
        {
          int cellIndex = cellItem.Index;

          //parse content
          FrameworkElement element = cellItem.CreateFormattedSketchControl();

          //if the element has a specified width, left-align the element
          if (cellItem.HasDirective(Directive.Width))
          {
            element.HorizontalAlignment = HorizontalAlignment.Left;
          }

          var margin = element.Margin;
          margin.Left += 2;
          margin.Right += 1;
          margin.Top += 1;
          margin.Bottom += 1;
          element.Margin = margin;

          if (ShowTableLines)
          {
            //element.VerticalAlignment = VerticalAlignment.Center;
            element = element.PutIntoBorder(0.5);
          }

          grid.Children.Add(element);

          //place element in cell
          Grid.SetColumn(element, cellIndex);
          Grid.SetRow(element, rowIndex);

          //let cell span to next occupied column if column isn't auto size
          if (!grid.ColumnDefinitions[cellIndex].Width.IsAuto)
          {
            //get the next cell, if there is one
            var nextCell = cells.FirstOrDefault(c => c.Index > cellIndex);
            int nextCellIndex = nextCell == null ? columnCount : nextCell.Index;

            Grid.SetColumnSpan(element, nextCellIndex - cellIndex);
          }
        }
      }

      return grid;
    }
  }
}
