﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System.Windows;
using System.Windows.Controls;

namespace Sketchables.Controls
{
  /// <summary>
  /// A column that renders cell values for <see cref="SketchItem"/>
  /// instances.
  /// </summary>
  public partial class DataGridSketchColumn : DataGridColumn
  {
    private static readonly DataGridLengthConverter ColumnLengthConverter = new DataGridLengthConverter();

    /// <summary>
    /// The column index, which is used to resolve the right cell
    /// item from the bound <see cref="RowSketchItem"/>.
    /// </summary>
    public int ColumnIndex { get; set; }

    /// <summary>
    /// The sketch item that represents this specific column.
    /// </summary>
    public SketchItem ColumnItem { get; set; }



    /// <summary>
    /// Creates a new column instance and formats it accordingly to
    /// its directives.
    /// </summary>
    /// <param name="columnItem"></param>
    /// <returns></returns>
    public static DataGridSketchColumn Create(SketchItem columnItem)
    {
      var col = new DataGridSketchColumn
                  {
                    ColumnIndex = columnItem.Index,
                    Header = columnItem.ContentMarkup,
                    ColumnItem = columnItem
                  };


      var directive = columnItem.TryGetDirective("w") ?? new Directive("w:*");
      col.Width = (DataGridLength) ColumnLengthConverter.ConvertFromString(directive.Value);

      return col;
    }



    /// <summary>
    /// When overridden in a derived class, gets a read-only element that is bound to the column's <see cref="P:System.Windows.Controls.DataGridBoundColumn.Binding"/> property value.
    /// </summary>
    /// <returns>
    /// A new, read-only element that is bound to the column's <see cref="P:System.Windows.Controls.DataGridBoundColumn.Binding"/> property value.
    /// </returns>
    /// <param name="cell">The cell that will contain the generated element.</param><param name="dataItem">The data item represented by the row that contains the intended cell.</param>
    protected override FrameworkElement GenerateElement(DataGridCell cell, object dataItem)
    {
      var cellItem = GetCellItem(dataItem);
      if (cellItem == null) return null;

      var control = cellItem.CreateFormattedSketchControl();
      
      //if the column is disabled, disable the control in any case, otherwise check the item's configuration
      if (ColumnItem != null && !ColumnItem.IsEnabled())
      {
#if SILVERLIGHT
      	var ctrl = control as Control;
				if (ctrl != null) ctrl.IsEnabled = false;
#else
        control.IsEnabled = false;
#endif
      }
      return control;
    }



    /// <summary>
    /// When overridden in a derived class, gets an editing element that is bound to the column's <see cref="P:System.Windows.Controls.DataGridBoundColumn.Binding"/> property value.
    /// </summary>
    /// <returns>
    /// A new editing element that is bound to the column's <see cref="P:System.Windows.Controls.DataGridBoundColumn.Binding"/> property value.
    /// </returns>
    /// <param name="cell">The cell that will contain the generated element.</param><param name="dataItem">The data item represented by the row that contains the intended cell.</param>
    protected override FrameworkElement GenerateEditingElement(DataGridCell cell, object dataItem)
    {
      var cellItem = GetCellItem(dataItem);
      string text = cellItem == null ? "" : cellItem.ContentMarkup;
      return ControlBuilder.CreateTextBox(text);
    }


    /// <summary>
    /// Gets the sketch item that is represented by the currently processed cell.
    /// </summary>
    /// <param name="dataItem">The <see cref="RowSketchItem"/> that contains all
    /// cell values for the given row.</param>
    /// <returns></returns>
    private SketchItem GetCellItem(object dataItem)
    {
      var sketchItem = dataItem as RowSketchItem;
      if(sketchItem == null || sketchItem.Cells.Count <= ColumnIndex)
      {
        return null;
      }
      return sketchItem.Cells[ColumnIndex];
    }
  }
}
