// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace Sketchables.Controls
{
  /// <summary>
  /// Creates a data grid control.
  /// </summary>
  public partial class SketchableDataGrid : RuntimeCompositionSketchControl, ISelectableSketchControl
  {
    /// <summary>
    /// The control view model, which provides the data context
    /// of the rendered grid.
    /// </summary>
    public DataGridViewModel ControlViewModel { get; set; }

    protected DataGrid currentGrid;


    #region IsHeaderVisible

    /// <summary>
    /// IsHeaderVisible Dependency Property
    /// </summary>
    public static readonly DependencyProperty IsHeaderVisibleProperty =
        DependencyProperty.Register("IsHeaderVisible", typeof(bool), typeof(SketchableDataGrid),
            new PropertyMetadata(true, OnIsHeaderVisibleChanged));

    /// <summary>
    /// Gets or sets the IsHeaderVisible property. This dependency property 
    /// indicates ....
    /// </summary>
    public bool IsHeaderVisible
    {
      get { return (bool)GetValue(IsHeaderVisibleProperty); }
      set { SetValue(IsHeaderVisibleProperty, value); }
    }

    /// <summary>
    /// Handles changes to the IsHeaderVisible property.
    /// </summary>
    private static void OnIsHeaderVisibleChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var target = (SketchableDataGrid)d;
      var oldIsHeaderVisible = (bool)e.OldValue;
      bool newIsHeaderVisible = target.IsHeaderVisible;
      target.OnIsHeaderVisibleChanged(oldIsHeaderVisible, newIsHeaderVisible);
    }

    /// <summary>
    /// Provides derived classes an opportunity to handle changes to the IsHeaderVisible property.
    /// </summary>
    protected virtual void OnIsHeaderVisibleChanged(bool oldIsHeaderVisible, bool newIsHeaderVisible)
    {
      if(ControlViewModel != null && RuntimeElement != null)
      {
        ControlViewModel.SetHeaderVisibility(newIsHeaderVisible);
        ConfigureGrid((DataGrid) RuntimeElement, ControlViewModel);
      }
    }

    #endregion


    /// <summary>
    /// This abstract builder method is responsible for creating
    /// <see cref="SketchableControl.RuntimeElement"/> that is displayed
    /// at runtime and represents the sketched item.
    /// Do not mistake this method with the <see cref="SketchableControl.RefreshRuntimeElement"/>
    /// method which updates the UI!
    /// </summary>
    /// <returns></returns>
    protected override UIElement CreateRuntimeElementInternal()
    {
      string content = Content ?? "";

      //generate rows
      var rows = content.Replace("\r\n", "\n")
        .Split('\n')
        .Where(s => !String.IsNullOrWhiteSpace(s))
        .Select(RowSketchItem.Create)
        .ToList();

      if(rows.Count == 0)
      {
        return null;
      }

      //the first row provides the column definitions, whether we show the header or
      //not
      var headerRow = rows[0];
      
      //remove header row and bind remaining data rows to grid
      rows.RemoveAt(0);

      //clear old VM
      if(ControlViewModel != null)
      {
        ControlViewModel.SelectedItemEventInvoker = null;
      }

      //init VM
      var vm = new DataGridViewModel
                 {
                   RowItems = rows,
                   HeaderRow =  headerRow
                 };
      vm.SetHeaderVisibility(IsHeaderVisible);

      //store VM reference and assign it to the grid
      ControlViewModel = vm;
      
      //setup the grid control
      DataGrid grid = CreateDataGridControl();
      foreach (var columnItem in headerRow.Cells)
      {
        var column = DataGridSketchColumn.Create(columnItem);
        grid.Columns.Add(column);
      }
      
      ConfigureGrid(grid, vm);

      //register event invoker
      ControlViewModel.SelectedItemEventInvoker = RaiseItemSelected;

      return grid;
    }


    /// <summary>
    /// Raises the <see cref="SketchItemChanged"/> event.
    /// </summary>
    /// <param name="item">The activated item itself.</param>
    protected void RaiseItemSelected(SketchItem item)
    {
      EventHandler<SketchItemEventArgs> handler = SketchItemChanged;
      if (handler != null)
      {
        handler(this, new SketchItemEventArgs(item));
      }
    }


    /// <summary>
    /// An event that fires if an item was activated.
    /// </summary>
    public event EventHandler<SketchItemEventArgs> SketchItemChanged;


    /// <summary>
    /// Gets a list of all items that may be selected.
    /// </summary>
    public IEnumerable<SketchItemDescription> ItemDescriptions
    {
      get
      {
        if (ControlViewModel == null) return new SketchItemDescription[0];

        return ControlViewModel.RowItems
          .OrderBy(i => i.Index).Select(i => new SketchItemDescription
                                               {
                                                 Name = i.ContentMarkup,
                                                 Index = i.Index
                                               })
          .ToArray();
      }
    }

  }
}
