﻿// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;


namespace Sketchables.Controls
{
  /// <summary>
  /// Root base class of the Framework.
  /// </summary>
  [DefaultBindingProperty("Content")]
  public abstract class SketchableControl : Control
  {
    #region RuntimeElement


#if SILVERLIGHT
    public static readonly DependencyProperty RuntimeElementProperty =
        DependencyProperty.Register("RuntimeElement", typeof(UIElement), typeof(SketchableControl),
        new PropertyMetadata(null));
#else
    public static readonly DependencyProperty RuntimeElementProperty =
    DependencyProperty.Register("RuntimeElement", typeof(UIElement), typeof(SketchableControl),
    new FrameworkPropertyMetadata(null));
#endif

    /// <summary>
    /// Gets or sets the runtime element, which is produced based on the
    /// sketch <see cref="Content"/>.
    /// </summary>
    public UIElement RuntimeElement
    {
      get { return (UIElement)GetValue(RuntimeElementProperty); }
      set { SetValue(RuntimeElementProperty, value); }
    }

    #endregion


    #region Content

    /// <summary>
    /// Content Dependency Property
    /// </summary>
    public static readonly DependencyProperty ContentProperty =
        DependencyProperty.Register("Content", typeof(string), typeof(SketchableControl),
            new PropertyMetadata(String.Empty, OnContentChanged));


    /// <summary>
    /// A string of sketching markup. This markup is used to create the
    /// <see cref="RuntimeElement"/>.
    /// </summary>
    public string Content
    {
      get { return (string)GetValue(ContentProperty); }
      set { SetValue(ContentProperty, value); }
    }

    /// <summary>
    /// Handles changes to the Content property.
    /// </summary>
    private static void OnContentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var owner = (SketchableControl)d;
      owner.OnSketchContentChanged((string) e.OldValue, (string) e.NewValue);
    }

    #endregion


    /// <summary>
    /// Invoked by the base class whenever the <see cref="Content"/> property changes.
    /// Deriving classes can override this method if more control over the composition
    /// process is required.
    /// </summary>
    protected virtual void OnSketchContentChanged(string oldValue, string newValue)
    {
      RefreshRuntimeElement();
    }


    /// <summary>
    /// This method can be invoked in order to update or swap out the
    /// current <see cref="RuntimeElement"/>.
    /// </summary>
    public abstract void RefreshRuntimeElement();


    protected SketchableControl()
    {
      DefaultStyleKey = typeof(SketchableControl);
    }

  }
}
