// Sketchables.NET - Rapid SketchFlow Prototyping
// (c) Copyright 2010 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE


using System;
using System.Collections.Generic;
using System.Linq;


namespace Sketchables.Controls
{
  /// <summary>
  /// Base class for controls that maintain a list of child items.
  /// This base class automatically splits the sketch markup and
  /// updates its <see cref="SketchItems"/> collection accordingly before
  /// triggering a refresh.
  /// </summary>
  public abstract class SketchableItemsControl : RuntimeCompositionSketchControl, ISelectableSketchControl
  {
    /// <summary>
    /// The list of items that were determined by splitting the
    /// Content string.
    /// </summary>
    public IEnumerable<SketchItem> SketchItems { get; set; }


    /// <summary>
    /// Parses the new content and updates the <see cref="SketchItems"/> collection
    /// accordingly.
    /// </summary>
    protected override void OnSketchContentChanged(string oldValue, string newValue)
    {
      IEnumerable<SketchItem> items = newValue.SplitMarkupString();

      SketchItems = items;
      base.OnSketchContentChanged(oldValue, newValue);
    }



    /// <summary>
    /// Raises the <see cref="SketchItemChanged"/> event.
    /// </summary>
    /// <param name="item">The activated item itself.</param>
    protected void RaiseItemChanged(SketchItem item)
    {
      EventHandler<SketchItemEventArgs> handler = SketchItemChanged;
      if (handler != null)
      {
        handler(this, new SketchItemEventArgs(item));
      } 
    }

    /// <summary>
    /// An event that fires if an item was activated.
    /// </summary>
    public event EventHandler<SketchItemEventArgs> SketchItemChanged;

    /// <summary>
    /// Gets a list of all items that may be selected.
    /// </summary>
    public virtual IEnumerable<SketchItemDescription> ItemDescriptions
    {
      get
      {
        if (SketchItems == null) return new SketchItemDescription[0];

        return SketchItems.OrderBy(i => i.Index)
          .Select(i => new SketchItemDescription
                         {
                           Name = i.ContentMarkup,
                           Index = i.Index
                         })
          .ToArray();
      }
    }
  }
}
