﻿using System;
using System.Linq;

namespace Sketchables
{
  public static class ItemExtensions
  {
    /// <summary>
    /// The item is checked or selected. Selection directive
    /// is <c>[x]</c>.
    /// </summary>
    public static bool IsSelected(this SketchItem item)
    {
      return item.HasDirective(Directive.Selected);
    }


    /// <summary>
    /// The item is disabled. Disabled directive
    /// is <c>[-]</c>.
    /// </summary>
    public static bool IsEnabled(this SketchItem item)
    {
      return !item.HasDirective(Directive.Disabled);
    }


    /// <summary>
    /// The item is a label that shouldn't be treated like regular
    /// items. Label directive is <c>[t]</c>.
    /// </summary>
    public static bool IsLabel(this SketchItem item)
    {
      return item.HasDirective(Directive.Label);
    }


    /// <summary>
    /// Checks whether the item's <see cref="SketchItem.ContentMarkup"/>
    /// is a formatting directive, which is a string that is surrounded
    /// by curly braces, e.g. <c>{@:http://www.google.com}</c>.
    /// </summary>
    /// <returns></returns>
    public static bool IsFormattingDirective(this SketchItem item)
    {
      if (String.IsNullOrWhiteSpace(item.ContentMarkup)) return false;
      var content = item.ContentMarkup.Trim();
      return content.StartsWith(Directive.FormatDirectivePrefix) && content.EndsWith(Directive.FormatDirectiveSuffix);
    }



    /// <summary>
    /// Updates <see cref="SketchItem.Value"/> and <see cref="SketchItem.Directives"/>
    /// of the item with an attribute directive.
    /// </summary>
    public static void AddAttributeDirective(this SketchItem item, Directive directive)
    {
      if(!item.Value.Trim().StartsWith(Directive.AttributeDirectivesPrefix))
      {
        //add a directive in front
        item.Value = String.Format("{0}{1}{2}", Directive.AttributeDirectivesPrefix, Directive.AttributeDirectivesSuffix, item.Value);        
      }

      //insert directive
      int directivesEnd = item.Value.IndexOf(Directive.AttributeDirectivesSuffix);
      item.Value = item.Value.Insert(directivesEnd, directive.ToString());

      //replace directives collection
      var directives = item.Directives.ToList();
      directives.Add(directive);
      item.Directives = directives;
    }
  }
}
