﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Sketchables
{
  /// <summary>
  /// Provides text parsing helper methods.
  /// </summary>
  public static class Parser
  {
    /// <summary>
    /// Regex used to split string by commas and/or line breaks.
    /// </summary>
    private static readonly Regex SplitStringRegex = new Regex(@"(?<!\\)[,\n]");

    /// <summary>
    /// The separator character used to split different directives.
    /// </summary>
    public const char DirectiveSeparator = '|';

    /// <summary>
    /// Splits a given string by either unescaped commas or line breaks.
    /// </summary>
    /// <param name="content"></param>
    /// <returns></returns>
    public static IEnumerable<string> SplitString(this string content)
    {
      if (String.IsNullOrWhiteSpace(content)) return new string[0];
      content = content.Replace("\r\n", "\n");
      return SplitStringRegex.Split(content);
    }


    /// <summary>
    /// Inits a given item by assigning its <see cref="SketchItem.Value"/> and
    /// <see cref="SketchItem.Index"/> properties, and splitting the the value's
    /// content and directives. Note that the assigned <see cref="SketchItem.ContentMarkup"/>
    /// will not be escaped or trimmed yet.
    /// </summary>
    /// <param name="item"></param>
    /// <param name="value"></param>
    /// <param name="index"></param>
    /// <param name="directiveStartChar"></param>
    /// <param name="directiveEndChar"></param>
    public static T Init<T>(this T item, string value, int index, string directiveStartChar = Directive.AttributeDirectivesPrefix,
                                                        string directiveEndChar = Directive.AttributeDirectivesSuffix)
      where T : SketchItem
    {
      item.Value = value;
      item.Index = index;

      string content;
      item.Directives = ExtractDirectives(value, out content, directiveStartChar, directiveEndChar);
      item.ContentMarkup = content;

      return item;
    }


    /// <summary>
    /// Ignores (trims) single spaces around entries, but treats others like regular placeholders.
    /// </summary>
    /// <param name="value"></param>
    /// <returns></returns>
    public static string TrimSingleSpace(this string value)
    {
      if (value == null) return value;

      //remove first and last space
      if (value.StartsWith(" ")) value = value.Substring(1, value.Length - 1);
      if (value.EndsWith(" ")) value = value.Substring(0, value.Length - 1);
      return value;
    }


    /// <summary>
    /// Checks whether a given string is inclosed in a given character set.
    /// </summary>
    /// <param name="value"></param>
    /// <param name="start"></param>
    /// <param name="end"></param>
    /// <returns></returns>
    public static bool IsEnclosedIn(this string value, string start, string end)
    {
      if (value == null) return false;
      string trimmed = value.Trim();
      return trimmed.StartsWith(start) && trimmed.EndsWith(end);
    }

    /// <summary>
    /// Finds directives at the beginning of a given markup string,
    /// and return the remaining content. 
    /// </summary>
    /// <param name="markup">The full markup string with directives and content.</param>
    /// <param name="content">The returned content. If the markup contains directives,
    /// these will be stripped. Note that the returned content may need to be unescaped.</param>
    /// <param name="directiveStartChar">The character that opens the directive section.</param>
    /// <param name="directiveEndChar">The character that defines closes the directive section.</param>
    public static IEnumerable<Directive> ExtractDirectives(string markup, out string content, string directiveStartChar,
                                                        string directiveEndChar)
    {
      string value = markup.TrimStart();

      //get all directives. These are the comma separated values in the
      //square brackets at the beginning of the sketch value, e.g. "[x|y] ABC"
      int directivesEnd = value.StartsWith(directiveStartChar) ? value.IndexOf(directiveEndChar) : -1;
      if (directivesEnd > -1)
      {
        content = value.Substring(directivesEnd + 1);

        return value.Substring(1, directivesEnd - 1)
          .Split(DirectiveSeparator)
          .Select(s => new Directive(s.Trim()))
          .ToArray();
      }

      //we don't have directives - return the unchanged markup
      content = markup;
      return new Directive[0];
    }


    /// <summary>
    /// Unescapes a given string in order 
    /// </summary>
    /// <param name="text"></param>
    /// <returns></returns>
    public static string Unescape(this string text)
    {
      var builder = new StringBuilder(text.Length);

      for (int charIndex = 0; charIndex < text.Length; charIndex++)
      {
        if (text[charIndex] == '\\')
        {
          //if we have an escape character, is should be followed by the escaped char
          if (charIndex + 1 == text.Length) throw new FormatException("String terminated by escape character.");

          //skip to the next (escaped) character
          charIndex++;

          //unescape the character and append it to the string
          switch (text[charIndex])
          {
            case 'n':
              builder.Append("\n");
              break;
            case 't':
              builder.Append("\t");
              break;
            case '\\':
              builder.Append(@"\");
              break;
            case ',':
              builder.Append(",");
              break;
            case '[':
              builder.Append("[");
              break;
            case '{':
              builder.Append("{");
              break;
            default:
              throw new FormatException("Unnown character after escape char: " + text[charIndex]);
          }
        }
        else
        {
          //just append regular chars
          builder.Append(text[charIndex]);
        }
      }

      return builder.ToString();
    }


    /// <summary>
    /// Escapes a given string.
    /// </summary>
    /// <param name="text"></param>
    /// <returns></returns>
    public static string Escape(this string text)
    {
      var builder = new StringBuilder(text.Length + 10);

      foreach (char character in text)
      {
        builder.Append(EscapeCharacter(character));
      }

      return builder.ToString();
    }


    private static string EscapeCharacter(char character)
    {
      switch (character)
      {
        case '\n':
          return @"\n";
        case '\t':
          return @"\t";
        case '\\':
          return @"\\";
        case ',':
          return @"\,";
        case '[':
          return @"\[";
        case '{':
          return @"\{";
      }

      return character.ToString();
    }
  }
}